############################################################
#                                                          #
#              Information                                 #
#                                                          #
############################################################

#Dates: 1/01/2021 24h *de moment*   
#Time zone and hour format: Hour of the day is shown in UTC. Value at Hour 1 correspond to the hour starting at 1 and ending at 2. As April falls in summer time (UTC + 2 hours), to convert from local time to UTC we move two hours backwards. So, 7 am local time is 5 am UTC.   
#Pollutant: NO2   
#Catalunya    
#Sites: 72 monitoring stations
#Observations source: 
#Models intercompared:  
#  Raw model from Caliope
#  LOOCV applied to Caliope
#-------------------------------------------------------------------------------
# Measure the time it takes to run the analysis
start_time <- Sys.time()
#Images 1 and 2 are the path where we want to save the graphics obtained in time series and temporal variations
Images   <- "/esarchive/scratch/mhajji/Meriem/UK_regional/proves_LOOCV/"
Images1  <- "/esarchive/scratch/mhajji/Meriem/UK_regional/proves_LOOCV/time_series_LOOCV/"
Images2  <- "/esarchive/scratch/mhajji/Meriem/UK_regional/proves_LOOCV/temporal_variation_LOOCV/"
#-------------------------------------------------------------------------------
#              LOOCV TO CALIOPE DATA WITH CATALONIA OBSERVATIONS
#-------------------------------------------------------------------------------

############################################################
#                                                          #
#              Set libraries                               #
#                                                          #
############################################################
# ! check the version of the package or R if any library or function does not work correctly

library(urbanR)
library(RNetCDF)
library(sp)
library(sf)
#library(mapview)
library(ncdf4)
library(rgeos) 
library(ggplot2)
library(plyr) #OJO! la llibreria plyr ha de ser cridada abans que dplyr 
library(dplyr)
library(broom)
library(magrittr)
library(ggpubr)
library(raster)
library(rgdal)
#library(tmap)
library(gstat)
library(sp)
library(abind)
library(openair)
#library(Metrics)
#-------------------------------------------------------------------------------

## LOAD config_file

args = commandArgs(TRUE)
if (length(args) > 0) {
  configFile_path <- args[1]
} else {
  cat("\n***WARNING: no ConfigFile was provided. Using default ConfigFile.\n")
}

#config_file it's a file where we change the data, where we want to save the results, etc. so now we extract these data from it:
#configFile_path <- "/esarchive/scratch/mhajji/Meriem/RUN_UK_regional/LOOCV/config_file_2021.yml"

config          <- yaml::yaml.load_file(configFile_path)
sim_folder_path <- config$sim_folder_path
obs_path        <- config$obs_path
model_path      <- config$model_path
save_result     =  config$save_result
date_time_end   <- config$date_time_end
date_time_begin <- config$date_time_begin
variogram_path  =  config$variogram_path
name            =  config$doc_name

############################################################
#                                                          #
#              Set temporal context                        #
#                                                          #
############################################################

#date_times et fa un super vector d'una  columna, dia per dia, hora per hora des de l'inici fins a la data final p.ex: 2021-01-01 00:00:00
date_times <- data_frame_dates(date_time_begin = date_time_begin,
                               date_time_end = date_time_end)

#dates: Llista de caracters anymesdia p.ex: 20210101
dates <- unique(paste0(as.character(strftime(date_times[, 1], format="%Y", tz = "UTC")),
                       as.character(strftime(date_times[, 1], format="%m", tz = "UTC")),
                       as.character(strftime(date_times[, 1], format="%d", tz = "UTC"))))
#year_month: igual que date pero anymes p.ex: 202101
year_month <- unique(paste0(as.character(strftime(date_times[, 1], format="%Y", tz = "UTC")),
                            as.character(strftime(date_times[, 1], format="%m", tz = "UTC"))))

############################################################
#                                                          #
#                 Load Observations                        #
#                                                          #
############################################################


# local_stations: special points data frame with two cols [NO2 and ref_obs-reference of station-], at the moment we have 72 stations in Cat
local_stations     <- readRDS(file= paste0(sim_folder_path,"local_stations.rds"))
df_obs_all         <- readRDS(file= paste0("/esarchive/scratch/mhajji/Meriem/RUN_UK_regional/LOOCV/","df_obs.rds"))
llista_uk_total_df <- readRDS(file= paste0("/esarchive/scratch/mhajji/Meriem/RUN_UK_regional/LOOCV/","llista_uk_total_df.rds"))
LOOCV_total_df     <- readRDS(file= paste0("/esarchive/scratch/mhajji/Meriem/RUN_UK_regional/LOOCV/","LOOCV_total_df.rds"))


############################################################
#                                                          #
#              evaluation statistics                       #
#                                                          #
############################################################

# Subset observations

#time_subset permits us to select the observations at the same dates as the model (remember: at first we extract observations monthly)
df_obs_sub <- time_subset(data = df_obs_all, time_var_name = "date",
                          date_time_begin = date_time_begin,
                          date_time_end = date_time_end)

# it alerts us if the dates in model do not coincide with the dates of the obs
if ( any(date_times$date != df_obs_sub$date))  {
  print("ERROR dates from model and observations are wrong")
}

#-------------------------------------------------------------------------------
# We need a data.frame with raw_model and LOOCV_model to apply the stadistics

# Comparation using all stations concatenated:
raw_model_total       <- NULL
corr_model_total      <- NULL 

for (i in 1:nrow(local_stations)) {

  raw_model_estacio <- data.frame(obs = df_obs_sub[ ,colnames(llista_uk_total)[i]],
                           mod = llista_uk_total_df[ ,colnames(llista_uk_total)[i]],
                           model="raw_model")
  
  corr_model_estacio <- data.frame(obs = df_obs_sub[ ,colnames(llista_uk_total)[i] ],
                          mod = LOOCV_total_df[ ,colnames(llista_uk_total)[i]],
                          model="corr_model")
  #raw_model_total and corr_model_total are a data frame with three lists (obs, mod, model(raw or corrected))
  raw_model_total   <- rbind(raw_model_total,raw_model_estacio) 
  corr_model_total  <- rbind(corr_model_total,corr_model_estacio)
}

mod_obs_data <- rbind(raw_model_total,
                      corr_model_total)

#Openair function that gives us the stadistics we want

Summary_stats <- openair::modStats(mod_obs_data, obs = "obs", mod = "mod", type = "model")
png(file = paste0(Images,"All_stations","_stats.png"), width = 1472, height = 540)
gridExtra::grid.table(Summary_stats)
dev.off()
#-------------------------------------------------------------------------------
# Mean value for each hour all stations:
obs_total_mean          <- matrix(nrow = 24, ncol = 0) 
raw_model_total_mean    <- matrix(nrow = 24, ncol = 0)
corr_model_total_mean   <- matrix(nrow = 24, ncol = 0)

for (i in 1:nrow(local_stations)) {
  obs_estacio        <-  df_obs_sub[ ,colnames(llista_uk_total)[i]]
  raw_model_estacio  <-  llista_uk_total_df[ ,colnames(llista_uk_total)[i]]
  corr_model_estacio <-  LOOCV_total_df[ ,colnames(llista_uk_total)[i]]

  obs_total_mean        <- cbind(obs_total_mean,obs_estacio)
  raw_model_total_mean  <- cbind(raw_model_total_mean,raw_model_estacio)
  corr_model_total_mean <- cbind(corr_model_total_mean,corr_model_estacio)
}

raw_model_mean      <- data.frame( obs = rowMeans(obs_total_mean,na.rm=TRUE),
                         mod = rowMeans(raw_model_total_mean,na.rm=TRUE),
                         model="raw_model")

corr_model_mean     <- data.frame( obs = rowMeans(obs_total_mean,na.rm=TRUE),
                          mod = rowMeans(corr_model_total_mean,na.rm=TRUE),
                          model="corr_model")

mod_obs_data_mean   <- rbind(raw_model_mean,
                       corr_model_mean)


Summary_stats <- openair::modStats(mod_obs_data_mean, obs = "obs", mod = "mod", type = "model")
print(Summary_stats)

png(file = paste0(Images,"All_stations_hourly","_stats.png"), width = 1472, height = 540)
gridExtra::grid.table(Summary_stats)
dev.off()


############################################################
#                                                          #
#                   Time series                            #
#                                                          #
############################################################

# Plot per cada estació:

month_names = c("January","February","March","April","May","June","July","August",
                 "September","October","November","December")


for (l in 1:nrow(local_stations)) {
  data_single     <- data.frame(date               = date_times,
                                observations       = df_obs_sub[ ,colnames(llista_uk_total)[l]],
                                Model              = llista_uk_total_df[ ,colnames(llista_uk_total)[l]], 
                                Model_corrected    = LOOCV_total_df[ ,colnames(llista_uk_total)[l]]) 
  for (mm in month_names) {
    # TIME_PLOT
    png(file = paste0(Images1,colnames(llista_uk_total)[l],"_",mm,"_24h.png"), width = 800, height = 400)
    openair::timePlot(selectByDate(data_single, year = 2021, month = mm),
                      pollutant = c("observations","Model","Model_corrected"),
                      group = TRUE,
                      title=colnames(llista_uk_total)[l],
                      y.relation = "free",
                      ylab="NO2 (ug/m3)",
                      cols = c("black", "blue", "grey"),
                      lty = c("dotted","solid","dashed"),
                      lwd = c(3,2,2), 
                      name.pol = c("Observations","Raw model", "Corrected Model"))
    dev.off()
}
}

#-------------------------------------------------------------------------------
# Plot per la mitjana horària de totes les estacions:

#First, we create a matrix to put the data of observations and model (corrected and raw)
obs_total            <- matrix(nrow = length(date_times$date), ncol = 0) 
raw_model_total      <- matrix(nrow = length(date_times$date), ncol = 0)
corr_model_total     <- matrix(nrow = length(date_times$date), ncol = 0)

for (i in 1:nrow(local_stations)) {
  obs_estacio           <-  df_obs_sub[ ,colnames(llista_uk_total)[i]]
  raw_model_estacio     <-  llista_uk_total_df[ ,colnames(llista_uk_total)[i]]
  corr_model_estacio    <-  LOOCV_total_df[ ,colnames(llista_uk_total)[i]]
  
  obs_total             <- cbind(obs_total,obs_estacio)
  raw_model_total       <- cbind(raw_model_total,raw_model_estacio)
  corr_model_total      <- cbind(corr_model_total,corr_model_estacio)
}

#Then, we calculate the means of each hour considering the whole stations
raw_model   <- data.frame( obs = rowMeans(obs_total,na.rm=TRUE),
                         mod = rowMeans(raw_model_total,na.rm=TRUE),
                         model="raw_model")

corr_model   <- data.frame( obs = rowMeans(obs_total,na.rm=TRUE),
                          mod = rowMeans(corr_model_total,na.rm=TRUE),
                          model="corr_model")

data_single   <- data.frame(date      = date_times,
                              observations       = raw_model$obs,
                              Model              = raw_model$mod,
                              Model_corrected    = corr_model$mod)

#Finally, we plot the time series 
# month_names = c("January","February","March","April","May","June","July","August",
#                 "September","October","November","December")

month_names = c("January","February","March","April","May","June","July","August",
                "September","October","November","December")

png(file = paste0(Images,"time_series_","hourly_mean","_",mm,"_24h.png"), width = 800, height = 400)
openair::timePlot(selectByDate(data_single, year = 2021, month = month_names),
                  pollutant = c("observations","Model","Model_corrected"),
                  group = TRUE,
                  y.relation = "free",
                  ylab="NO2 (ug/m3)",
                  cols = c("black", "blue", "grey"),
                  lty = c("dotted","solid","dashed"),
                  lwd = c(3,2,2),
                  name.pol = c("observations","Raw model", "Corrected Model"))
dev.off()

############################################################
#                                                          #
#              Temporal variations                         #
#                                                          #
############################################################
#-------------------------------------------------------------------------------
# Plot per la mitjana de totes les estacions:

#First, we create a matrix to put the data of observations and model (corrected and raw)
obs_total          <- matrix(nrow = length(date_times$date), ncol = 0) 
raw_model_total    <- matrix(nrow = length(date_times$date), ncol = 0)
corr_model_total   <- matrix(nrow = length(date_times$date), ncol = 0)

for (i in 1:nrow(local_stations)) {
  obs_estacio         <-  df_obs_sub[ ,colnames(llista_uk_total)[i]]
  raw_model_estacio   <-  llista_uk_total_df[ ,colnames(llista_uk_total)[i]]
  corr_model_estacio  <-  LOOCV_total_df[ ,colnames(llista_uk_total)[i]]
  
  obs_total        <- cbind(obs_total,obs_estacio)
  raw_model_total  <- cbind(raw_model_total,raw_model_estacio)
  corr_model_total <- cbind(corr_model_total,corr_model_estacio)
}

#Then, we calculate the means of each hour considering the whole stations
raw_model <- data.frame( obs = rowMeans(obs_total,na.rm=TRUE),
                         mod = rowMeans(raw_model_total,na.rm=TRUE),
                         model="raw_model")

corr_model <- data.frame( obs = rowMeans(obs_total,na.rm=TRUE),
                          mod = rowMeans(corr_model_total,na.rm=TRUE),
                          model="corr_model")

Mean_all_stations            <- data.frame(date            = date_times,
                                        observations       = raw_model$obs,
                                        Model              = raw_model$mod, 
                                        Model_corrected    = corr_model$mod)

TIME_VAR <- openair::timeVariation(Mean_all_stations,
                                   pollutant = c("observations","Model","Model_corrected"),
                                   ylab = "no2 (ug/m3)",
                                   type ="month",
                                   key = TRUE,
                                   title = "Media horaria Cat",
                                   auto.text=FALSE, 
                                   ylim = c(0, 120),
                                   cols = c("black", "blue", "grey"),
                                   lty = c("dotted","solid","dashed"),
                                   lwd = c(3,2,2), 
                                   name.pol = c("Observations", "Raw model", "Corrected Model"))  

png(file = paste0(Images,"temporal","_mean_time_VAR_24h.png"), width = 800, height = 600)
print(TIME_VAR[[1]][2]) 
dev.off()

#-------------------------------------------------------------------------------
# Plot per cada estació:

for (n in 1:nrow(local_stations)) {
  
  Single_station            <- data.frame(date               = date_times,
                                          observations       = df_obs_sub[ ,colnames(llista_uk_total)[n]],
                                          Model              = llista_uk_total_df[ ,colnames(llista_uk_total)[n]], 
                                          Model_corrected    = LOOCV_total_df[ ,colnames(llista_uk_total)[n]])
  
  TIME_VAR <- openair::timeVariation(Single_station,
                                     pollutant = c("observations","Model","Model_corrected"),
                                     ylab = "no2 (ug/m3)",
                                     type ="month",
                                     key = TRUE,
                                     title = colnames(llista_uk_total)[n],
                                     auto.text=FALSE, 
                                     ylim = c(0, 120),
                                     cols = c("black", "blue", "grey"),
                                     lty = c("dotted","solid","dashed"),
                                     lwd = c(3,2,2), 
                                     name.pol = c("observations","Raw model", "Corrected Model"))  
  
  png(file = paste0(Images2,colnames(llista_uk_total)[n],"_time_VAR_24h.png"), width = 800, height = 600)
  print(TIME_VAR[[1]][2]) 
  dev.off()
  
}

#-------------------------------------------------------------------------------
#Temps que triga en executar-se el fitxer

end_time <- Sys.time()
temps_total <- end_time-start_time
print(temps_total)

